#ifndef _MPLUGIN_H_
#define _MPLUGIN_H_

// Required includes
#include <extdll.h>
#include <meta_api.h>
#include "mglobals.h"

// Some compiler specific macros
#ifdef _WIN32
#define C_EXPORT extern "C" __declspec( dllexport )
#define GIVEFNPT extern "C" __declspec( dllexport ) void __stdcall GiveFnptrsToDll

// For exporting GiveFnptrsToDll on win32 using msvc without a .def file
// Thanks to koraX for this info!
#pragma comment( linker, "/export:GiveFnptrsToDll=_GiveFnptrsToDll@8,@1" )
#pragma comment( linker, "/section:.data,RW" )
#else
#define C_EXPORT extern "C"
#define GIVEFNPT extern "C" void GiveFnptrsToDll
#endif

// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
// !!!! CHANGE THESE TO SUIT YOUR PLUGIN !!!!
// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
#define THISPLUGIN_AUTHOR "Lazy"
#define THISPLUGIN_CONTACT "no_contact"
#define THISPLUGIN_VERSION "1.0.0"
#define THISPLUGIN_LOGTAG "MMCS"
#define THISPLUGIN_NAME "Map/Model/Sound Changer"
#define THISPLUGIN_LOADABLE PT_ANYTIME
#define THISPLUGIN_UNLOADABLE PT_ANYTIME

// Comment/uncomment whatever main APIs you want to hook
// TODO:
// Add all of them
#define MHOOK_GETENGINEFUNCTIONS
//#define MHOOK_GETENGINEFUNCTIONS_POST
#define MHOOK_GETENTITYAPI2
//#define MHOOK_GETENTITYAPI2_POST

// To macros which attempt to reduce typing when hooking
// various functions
#define HOOK_DLLAPI_FN( dllapifunc, hook ) gFunctionTable.dllapifunc = hook
#define HOOK_ENGINE_FN( engapifunc, hook ) meta_engfuncs.engapifunc = hook

// Macro for compilation date and time
// Needed because gcc does not support __TIMESTAMP__
#define _COMPILETIME_ __DATE__ " " __TIME__

// Called when metamod is loading this plugin
// After this macro, you can put code to be executed before the function returns
#define BEGIN_META_ATTACH( ) \
C_EXPORT int Meta_Attach( PLUG_LOADTIME, META_FUNCTIONS* pMetafunctiontable, meta_globals_t* pMetaglobals, gamedll_funcs_t* pGamedllfuncs ) { \
   if ( ! pMetafunctiontable || ! pMetaglobals || ! pGamedllfuncs ) \
      return 0; \
      \
      memcpy( pMetafunctiontable, &gMetaFunctionTable, sizeof( META_FUNCTIONS ) ); \
      gpMetaGlobals = pMetaglobals; \
      gpGamedllFuncs = pGamedllfuncs; \
      \
      return 1;
      
// Closes off the Meta_Attach function
#define END_META_ATTACH( ) return 1; }

// Called when metamod is asking for our plugin_info_t struct.
#define BEGIN_META_QUERY( ) \
C_EXPORT int Meta_Query( char*, plugin_info_t** ppInfo, mutil_funcs_t* pMetautilfuncs ) { \
   *ppInfo = &Plugin_info; \
   gpMetaUtilFuncs = pMetautilfuncs;

// Finishes off the BEGIN_META_QUERY macro
#define END_META_QUERY( ) return 1; }

// Called when metamod is unloading/pausing the plugin
#define BEGIN_META_DETACH( ) \
C_EXPORT int Meta_Detach( PLUG_LOADTIME, PL_UNLOAD_REASON ) {

// Ending macro for BEGIN_META_ATTACH
#define END_META_DETACH( ) return 1; }

// Called when we receive the engine functions and global variables
#define BEGIN_GIVEFNPTRSTODLL( ) \
GIVEFNPT( enginefuncs_t* pEnginefuncs, globalvars_t* pGlobals ) { \
   memcpy( &g_engfuncs, pEnginefuncs, sizeof( enginefuncs_t ) ); \
   gpGlobals = pGlobals;

// Ending macro for GIVEFNPTRSTODLL
#define END_GIVEFNPTRSTODLL( ) }

#define BEGIN_GETENGINEFUNCTIONS( ) \
C_EXPORT int GetEngineFunctions( enginefuncs_t* pEnginefuncs, int* pInterfaceversion ) { \
   if ( ! pEnginefuncs ) \
      return 0; \
   if ( *pInterfaceversion != ENGINE_INTERFACE_VERSION ) { \
      *pInterfaceversion = ENGINE_INTERFACE_VERSION; \
      return 0; \
   } \
   \
   memset( &meta_engfuncs, 0, sizeof( enginefuncs_t ) );

#define END_GETENGINEFUNCTIONS( ) \
   memcpy( pEnginefuncs, &meta_engfuncs, sizeof( enginefuncs_t ) ); \
   return 1; \
}

#define BEGIN_GETENTITYAPI2( ) \
C_EXPORT int GetEntityAPI2( DLL_FUNCTIONS* pFunctiontable, int* pInterfaceversion ) { \
   if ( ! pFunctiontable ) \
      return 0; \
   if ( *pInterfaceversion != INTERFACE_VERSION ) { \
      *pInterfaceversion = INTERFACE_VERSION; \
      return 0; \
   } \
   memset( &gFunctionTable, 0, sizeof( DLL_FUNCTIONS ) );

#define END_GETENTITYAPI2( ) \
   memcpy( pFunctiontable, &gFunctionTable, sizeof( DLL_FUNCTIONS ) ); \
   return 1; \
}

#endif
